
package rpcs.system;

import com.sun.java.swing.*;
import java.awt.*;
import java.util.*;
import rpcs.system.*;

public class RPCSMapArea extends JPanel {

	public static final int MAP_NONE = 0;
	public static final int MAP_HEX = 1;
	public static final int MAP_SQUARE = 2;

	public static final float   COS30 = (float )0.8660;
	public static final float   SIN30 = (float )0.5;
	public static final float   TAN30 = (float )0.5773;

	public final int MAPSIZE = 200;

	int GridType;

	int hex_hgt  = 52;
	int gridSize = 60;
	int OldgridSize = 60;
	int zoomStep = 0;

	int hex_mx0, hex_mx1, hex_mx2, hex_my0, hex_my1, 
	    hex_my2, hex_my3, hex_my4;

	public int hex_incx, hex_incy;

	RPCSParam Params;
	Rectangle MapButtonSize;

	Object[][][] MapObjects;

	public RPCSMapArea(RPCSParam params) {
		super();
		this.Params = params;

//		MapObjects = new Object[MAPSIZE][MAPSIZE][];

		gridSize = 60;

		setLayout(new PositionLayout());
		setSize(1000, 1000);
		setBackground(java.awt.Color.white);

		GridType = this.MAP_NONE;

		MapButtonSize = new Rectangle();

		initHex(60);
	}

	public void initHex(int size) {
		gridSize = size;
		hex_hgt = (int) (size * COS30);

		hex_incx = (int )(hex_hgt * COS30);
		hex_incy = (int )(hex_hgt * SIN30);

		hex_mx1 = (int )(hex_hgt * COS30);
		hex_mx2 = (int )(2 * hex_hgt * COS30);
		hex_my1 = 0;
		hex_my2 = (int )((hex_hgt * SIN30) + hex_my1);
		hex_my3 = (int )(hex_hgt + hex_my2);
		hex_my4 = (int )((hex_hgt * SIN30) + hex_my3);

		hex_mx0 = (int )((hex_mx1 - (gridSize/2)) + 1); 
		hex_my0 = (int )(((gridSize/2) * TAN30) + 1);
	}

	public Dimension getPreferredSize() {
		return new Dimension(1000,1000);
	}

	public void setGridType(int type) {
		GridType = type;
	}

	public void paint(Graphics gr) {
		Dimension dim = this.getSize();
		int x=0,y=0, i;

		super.paint(gr);

//		gr.setColor(Color.black);
//		gr.setColor(Color.lightGray);
		gr.setColor(Color.gray);
		if (GridType == MAP_HEX) {
		   y -= hex_incy + hex_hgt;
		   for (int j=0; j<28; j++)
		   {
	  		x = 0;
	  		i = 0;
	  		if ((j % 2) == 1)
	  		{
	    	  	   x += hex_incx;
			   i = 1;
	  		}
	  		y += hex_incy + hex_hgt;
	  		x -= (hex_incx * 2);
	  		for (; i<25; i++)
	  		{
	   		   x += (hex_incx * 2);
	   		   gr.drawLine(x+hex_mx1, y+hex_my1, x, y+hex_my2);
	   		   gr.drawLine(x+hex_mx1, y+hex_my1, x+hex_mx2, y+hex_my2);
	   		   gr.drawLine(x+hex_mx2, y+hex_my2, x+hex_mx2, y+hex_my3);
		   	   gr.drawLine(x+hex_mx2, y+hex_my3, x+hex_mx1, y+hex_my4);
		   	   gr.drawLine(x+hex_mx1, y+hex_my4, x, y+hex_my3);
	   		   gr.drawLine(x, y+hex_my3, x, y+hex_my2);
	  		}
		   }
		}
		else if (GridType == MAP_SQUARE) {
			for (i=0; i<dim.width; i+=gridSize)
				gr.drawLine(i, 0, i, dim.height);
			for (i=0; i<dim.height; i+=gridSize)
				gr.drawLine(0, i, dim.width, i);
		}
	}

	public void ZoomIn() {
		if ((gridSize + 15) > 90)
		   return;
		zoomStep++;
		doZoom(+15);
	}

	public void ZoomOut() {
		if ((gridSize - 15) < 30)
		   return;
		zoomStep--;
		doZoom(-15);
	}

	public void doZoom(int adj) {
		OldgridSize = gridSize;
		gridSize += adj;
		initHex(gridSize);

		sizeButtons(Params.aRPCSMain.vRPCSPlayer, adj);
		sizeButtons(Params.aRPCSMain.vRPCSMonster, adj);
		sizeButtons(Params.aRPCSMain.vRPCSTrap, adj);

		getBounds(MapButtonSize);
		repaint(MapButtonSize);
	}

	public void sizeButtons(Vector vec, int adj) {
	   RPCSBaseObject tObj;
	   Dimension dim = new Dimension(gridSize, gridSize);

		for (int i=vec.size()-1; i>=0; i--) {
		   tObj = (RPCSBaseObject ) vec.elementAt(i);
		   tObj.ObjButton.setPreferredSize(dim);
		   tObj.ObjButton.setSize(dim);
		   convertToGrid(tObj.ObjButton);
		}
	}

	public void convertToGrid(RPCSButton button) {
	   int x, y, gx=0, gy=0;

	   if (GridType == MAP_NONE)
		return;

	   if (button.GridX == -1) {
	   	x = button.getX() + (gridSize / 2);
	   	y = button.getY() + (gridSize / 2);
	   	if (GridType == MAP_HEX) {
	   	   gy = y / hex_my3;
	   	   if ((gy % 2) == 1)
		      x -= hex_incx;
	   	   gx = x / hex_mx2;
		}
	   	else if (GridType == MAP_SQUARE) {
		   gy = y / gridSize;
		   gx = x / gridSize;
		}
		button.setGridLocation(gx, gy);
// System.out.println(button.getText() + " NGx = " + gx + " NGy = " + gy + " X = " + x + " Y = " + y);
	   }
	   else {
		gx = button.GridX;
		gy = button.GridY;
	   }

	   if (GridType == MAP_SQUARE) {
		button.setLocation(gx*gridSize, gy *gridSize);
	   }
	   else if (GridType == MAP_HEX) {
		x = gx * hex_mx2;
		y = (gy * hex_my3) + hex_incy;
		if ((gy % 2) == 1) {
		   x += hex_incx;
		}
		if (zoomStep < 0) {
		  x -= gx;
		}
		gx = (hex_mx2 - gridSize) / 2;
		x += gx;
		y -= (gx / 2) - 2 - zoomStep;
		button.setLocation(x, y);
// System.out.println(button.getText() + " Gx = " + gx + " Gy = " + gy + " X = " + x + " Y = " + y);
	   }
	}

	public Component add(Component obj) {
	   convertToGrid((RPCSButton )obj);
	   return super.add(obj);
	}
}
